@@ -1,30 +1,23 @@
-# -*- coding: utf-8 -*-
-
 """Test support of the various forms of tabular data."""
 
-from __future__ import print_function
-from __future__ import unicode_literals
-from tabulate import tabulate
-from common import assert_equal, assert_in, raises, skip
-try:
-    from collections import UserDict
-except ImportError:
-    # Python2
-    from UserDict import UserDict
+from tabulate2 import tabulate
+from common import raises, skip
+
+from collections import UserDict
 
 
 def test_iterable_of_iterables():
-    "Input: an interable of iterables."
+    "Input: an iterable of iterables."
     ii = iter(map(lambda x: iter(x), [range(5), range(5, 0, -1)]))
     expected = "\n".join(
         ["-  -  -  -  -", "0  1  2  3  4", "5  4  3  2  1", "-  -  -  -  -"]
     )
     result = tabulate(ii)
-    assert_equal(expected, result)
+    assert expected == result
 
 
 def test_iterable_of_iterables_headers():
-    "Input: an interable of iterables with headers."
+    "Input: an iterable of iterables with headers."
     ii = iter(map(lambda x: iter(x), [range(5), range(5, 0, -1)]))
     expected = "\n".join(
         [
@@ -35,11 +28,11 @@ def test_iterable_of_iterables_headers():
         ]
     )
     result = tabulate(ii, "abcde")
-    assert_equal(expected, result)
+    assert expected == result
 
 
 def test_iterable_of_iterables_firstrow():
-    "Input: an interable of iterables with the first row as headers"
+    "Input: an iterable of iterables with the first row as headers"
     ii = iter(map(lambda x: iter(x), ["abcde", range(5), range(5, 0, -1)]))
     expected = "\n".join(
         [
@@ -50,7 +43,7 @@ def test_iterable_of_iterables_firstrow():
         ]
     )
     result = tabulate(ii, "firstrow")
-    assert_equal(expected, result)
+    assert expected == result
 
 
 def test_list_of_lists():
@@ -65,7 +58,7 @@ def test_list_of_lists():
         ]
     )
     result = tabulate(ll, headers=["string", "number"])
-    assert_equal(expected, result)
+    assert expected == result
 
 
 def test_list_of_lists_firstrow():
@@ -80,7 +73,7 @@ def test_list_of_lists_firstrow():
         ]
     )
     result = tabulate(ll, headers="firstrow")
-    assert_equal(expected, result)
+    assert expected == result
 
 
 def test_list_of_lists_keys():
@@ -90,7 +83,7 @@ def test_list_of_lists_keys():
         ["0    1      2", "---  ---  ---", "a    one    1", "b    two"]
     )
     result = tabulate(ll, headers="keys")
-    assert_equal(expected, result)
+    assert expected == result
 
 
 def test_dict_like():
@@ -106,8 +99,7 @@ def test_dict_like():
         ["  b    a", "---  ---", "101    0", "102    1", "103    2", "104"]
     )
     result = tabulate(dd, "keys")
-    print("Keys' order: %s" % dd.keys())
-    assert_in(result, [expected1, expected2])
+    assert result in [expected1, expected2]
 
 
 def test_numpy_2d():
@@ -126,7 +118,7 @@ def test_numpy_2d():
             ]
         )
         result = tabulate(na, ["a", "b", "c"])
-        assert_equal(expected, result)
+        assert expected == result
     except ImportError:
         skip("test_numpy_2d is skipped")
 
@@ -141,7 +133,7 @@ def test_numpy_2d_firstrow():
             ["  1    8    27", "---  ---  ----", " 64  125   216", "343  512   729"]
         )
         result = tabulate(na, headers="firstrow")
-        assert_equal(expected, result)
+        assert expected == result
     except ImportError:
         skip("test_numpy_2d_firstrow is skipped")
 
@@ -162,7 +154,7 @@ def test_numpy_2d_keys():
             ]
         )
         result = tabulate(na, headers="keys")
-        assert_equal(expected, result)
+        assert expected == result
     except ImportError:
         skip("test_numpy_2d_keys is skipped")
 
@@ -188,7 +180,7 @@ def test_numpy_record_array():
             ]
         )
         result = tabulate(na)
-        assert_equal(expected, result)
+        assert expected == result
     except ImportError:
         skip("test_numpy_2d_keys is skipped")
 
@@ -214,7 +206,7 @@ def test_numpy_record_array_keys():
             ]
         )
         result = tabulate(na, headers="keys")
-        assert_equal(expected, result)
+        assert expected == result
     except ImportError:
         skip("test_numpy_2d_keys is skipped")
 
@@ -240,7 +232,7 @@ def test_numpy_record_array_headers():
             ]
         )
         result = tabulate(na, headers=["person", "years", "cm"])
-        assert_equal(expected, result)
+        assert expected == result
     except ImportError:
         skip("test_numpy_2d_keys is skipped")
 
@@ -260,7 +252,7 @@ def test_pandas():
             ]
         )
         result = tabulate(df, headers=["string", "number"])
-        assert_equal(expected, result)
+        assert expected == result
     except ImportError:
         skip("test_pandas is skipped")
 
@@ -277,7 +269,7 @@ def test_pandas_firstrow():
             ["a    one      1.0", "---  -----  -----", "b    two      nan"]
         )
         result = tabulate(df, headers="firstrow")
-        assert_equal(expected, result)
+        assert expected == result
     except ImportError:
         skip("test_pandas_firstrow is skipped")
 
@@ -299,7 +291,7 @@ def test_pandas_keys():
             ]
         )
         result = tabulate(df, headers="keys")
-        assert_equal(expected, result)
+        assert expected == result
     except ImportError:
         skip("test_pandas_keys is skipped")
 
@@ -321,7 +313,7 @@ whom      how old    how tall
 ------  ---------  ----------
 Alice          23       169.5
 Bob            27       175"""
-        assert_equal(expected, result)
+        assert expected == result
     except ImportError:
         skip("test_sqlite3 is skipped")
 
@@ -345,7 +337,7 @@ whom      how old    how tall
 ------  ---------  ----------
 Alice          23       169.5
 Bob            27       175"""
-        assert_equal(expected, result)
+        assert expected == result
     except ImportError:
         skip("test_sqlite3_keys is skipped")
 
@@ -358,7 +350,7 @@ def test_list_of_namedtuples():
     lt = [NT(1, 2), NT(3, 4)]
     expected = "\n".join(["-  -", "1  2", "3  4", "-  -"])
     result = tabulate(lt)
-    assert_equal(expected, result)
+    assert expected == result
 
 
 def test_list_of_namedtuples_keys():
@@ -371,7 +363,7 @@ def test_list_of_namedtuples_keys():
         ["  foo    bar", "-----  -----", "    1      2", "    3      4"]
     )
     result = tabulate(lt, headers="keys")
-    assert_equal(expected, result)
+    assert expected == result
 
 
 def test_list_of_dicts():
@@ -380,7 +372,8 @@ def test_list_of_dicts():
     expected1 = "\n".join(["-  -", "1  2", "3  4", "-  -"])
     expected2 = "\n".join(["-  -", "2  1", "4  3", "-  -"])
     result = tabulate(lod)
-    assert_in(result, [expected1, expected2])
+    assert result in [expected1, expected2]
+
 
 def test_list_of_userdicts():
     "Input: a list of UserDicts."
@@ -388,7 +381,8 @@ def test_list_of_userdicts():
     expected1 = "\n".join(["-  -", "1  2", "3  4", "-  -"])
     expected2 = "\n".join(["-  -", "2  1", "4  3", "-  -"])
     result = tabulate(lod)
-    assert_in(result, [expected1, expected2])
+    assert result in [expected1, expected2]
+
 
 def test_list_of_dicts_keys():
     "Input: a list of dictionaries, with keys as headers."
@@ -400,7 +394,8 @@ def test_list_of_dicts_keys():
         ["  bar    foo", "-----  -----", "    2      1", "    4      3"]
     )
     result = tabulate(lod, headers="keys")
-    assert_in(result, [expected1, expected2])
+    assert result in [expected1, expected2]
+
 
 def test_list_of_userdicts_keys():
     "Input: a list of UserDicts."
@@ -412,7 +407,8 @@ def test_list_of_userdicts_keys():
         ["  bar    foo", "-----  -----", "    2      1", "    4      3"]
     )
     result = tabulate(lod, headers="keys")
-    assert_in(result, [expected1, expected2])
+    assert result in [expected1, expected2]
+
 
 def test_list_of_dicts_with_missing_keys():
     "Input: a list of dictionaries, with missing keys."
@@ -427,7 +423,7 @@ def test_list_of_dicts_with_missing_keys():
         ]
     )
     result = tabulate(lod, headers="keys")
-    assert_equal(expected, result)
+    assert expected == result
 
 
 def test_list_of_dicts_firstrow():
@@ -441,7 +437,7 @@ def test_list_of_dicts_firstrow():
         ["  BAR    FOO    baz", "-----  -----  -----", "    4      3      5"]
     )
     result = tabulate(lod, headers="firstrow")
-    assert_in(result, [expected1, expected2])
+    assert result in [expected1, expected2]
 
 
 def test_list_of_dicts_with_dict_of_headers():
@@ -455,7 +451,7 @@ def test_list_of_dicts_with_dict_of_headers():
         ["LETTERS      DIGITS", "---------  --------", "ABCDE         12345"]
     )
     result = tabulate(table, headers=headers)
-    assert_in(result, [expected1, expected2])
+    assert result in [expected1, expected2]
 
 
 def test_list_of_dicts_with_list_of_headers():
@@ -466,7 +462,7 @@ def test_list_of_dicts_with_list_of_headers():
         tabulate(table, headers=headers)
 
 
-def test_py27orlater_list_of_ordereddicts():
+def test_list_of_ordereddicts():
     "Input: a list of OrderedDicts."
     from collections import OrderedDict
 
@@ -474,4 +470,56 @@ def test_py27orlater_list_of_ordereddicts():
     lod = [od, od]
     expected = "\n".join(["  b    a", "---  ---", "  1    2", "  1    2"])
     result = tabulate(lod, headers="keys")
-    assert_equal(expected, result)
+    assert expected == result
+
+
+def test_py37orlater_list_of_dataclasses_keys():
+    "Input: a list of dataclasses with first item's fields as keys and headers"
+    try:
+        from dataclasses import make_dataclass
+
+        Person = make_dataclass("Person", ["name", "age", "height"])
+        ld = [Person("Alice", 23, 169.5), Person("Bob", 27, 175.0)]
+        result = tabulate(ld, headers="keys")
+        expected = "\n".join(
+            [
+                "name      age    height",
+                "------  -----  --------",
+                "Alice      23     169.5",
+                "Bob        27     175",
+            ]
+        )
+        assert expected == result
+    except ImportError:
+        skip("test_py37orlater_list_of_dataclasses_keys is skipped")
+
+
+def test_py37orlater_list_of_dataclasses_headers():
+    "Input: a list of dataclasses with user-supplied headers"
+    try:
+        from dataclasses import make_dataclass
+
+        Person = make_dataclass("Person", ["name", "age", "height"])
+        ld = [Person("Alice", 23, 169.5), Person("Bob", 27, 175.0)]
+        result = tabulate(ld, headers=["person", "years", "cm"])
+        expected = "\n".join(
+            [
+                "person      years     cm",
+                "--------  -------  -----",
+                "Alice          23  169.5",
+                "Bob            27  175",
+            ]
+        )
+        assert expected == result
+    except ImportError:
+        skip("test_py37orlater_list_of_dataclasses_headers is skipped")
+
+
+def test_list_bytes():
+    "Input: a list of bytes. (issue #192)"
+    lb = [["你好".encode()], ["你好"]]
+    expected = "\n".join(
+        ["bytes", "---------------------------", r"b'\xe4\xbd\xa0\xe5\xa5\xbd'", "你好"]
+    )
+    result = tabulate(lb, headers=["bytes"])
+    assert expected == result
